unit Mouse;

interface

uses Dos;

const
  moUnknown = -1;
  moAbsent = -2;
  moOk = 0;

  moLeft = 0;
  moRight = 1;
  moMiddle = 2;

  moPress = 0;
  moRelease = 1;

  MoError: Integer = MoUnknown;
  ButtNumb: Word = 0;
  OMoTick: LongInt = 0;
  DMoTick: LongInt = 10;

  MoHandl: Pointer = Ptr (0, 0);
  MoMask: Word = 1;

type
  GraphMask = array[0..31] of Word;
  PGraphMask = ^GraphMask;

procedure MoInit;
procedure MoShow;
procedure MoHide;
function  MoGetPage: Word;
procedure MoSetPage (PageNo: Word);
procedure MoGraph (PMask: PGraphMask; HotX, HotY: Word);
procedure MoRangeX (Left, Right: Word);
procedure MoRangeY (Up, Down: Word);
procedure MoState (var Buttons, PosX, PosY);
function  MoPressed (var Buttons, PosX, PosY): Word;
function  MoReleased (var Buttons, PosX, PosY): Word;
function  MoDClick (var PosX, PosY): Boolean;
procedure MoGetMove (var GetX, GetY);
procedure MoGoToXY (X, Y: Word);
function  MoBuffSize: Word;
procedure MoSaveStatus (Buff: Pointer);
procedure MoLoadStatus (Buff: Pointer);
procedure MoExchHandler (var HandlPtr, Mask);

implementation

procedure MoInit;
var
  Vector : Pointer;
  Regs : Registers;
begin
  If (MoError = MoUnknown) then begin
    GetIntVec($33, Vector);
    If ((Vector = nil) or (Word (Vector^) = $CF)) then begin
      MoError := MoAbsent;
      ButtNumb := 0;
    end
    else begin
      Regs.AX := 0;
      Regs.BX := 0;
      Intr ($33, Regs);
      If (Regs.AX=0) then begin
        MoError := MoAbsent;
        ButtNumb := 0
      end
      else begin
        MoError := MoOk;
        If (Regs.BX=0) then ButtNumb := 1;
        If (Regs.BX=2) then ButtNumb := 2;
        If (Regs.BX=3) then ButtNumb := 3;
      end;
    end;
  end;
end;

procedure MoGraph (PMask: PGraphMask; HotX, HotY: Word); assembler;
asm
  mov ax, 9
  mov bx, HotX
  mov cx, HotY
  les dx, PMask
  int 33h
end;

procedure MoRangeX(Left,Right : Word); assembler;
asm
  mov ax, 7
  mov cx, Left
  mov dx, Right
  int 33h
end;

procedure MoRangeY (Up, Down: Word); assembler;
asm
  mov ax, 8
  mov cx, Up
  mov dx, Down
  int 33h
end;

procedure MoState (var Buttons, PosX, PosY); assembler;
asm
  mov ax, 3
  int 33h
  les di, PosX
  mov word ptr es:[di], cx
  les di, PosY
  mov word ptr es:[di], dx
  les di, Buttons
  mov word ptr es:[di], bx
end;

function MoDClick (var PosX, PosY): Boolean;
var
  Count: Word;
  NMoTick: LongInt absolute $0040:$006C;
begin
  asm
    mov ax, 5
    mov bx, 0
    int 33h
    les di, PosX
    mov word ptr es:[di], cx
    les di, PosY
    mov word ptr es:[di], dx
    mov Count, bx
  end;
  if (NMoTick-OMoTick < DMoTick) and (Count > 0) then
    MoDClick := TRUE
  else
    MoDClick := FALSE;
  if (Count > 0) then OMoTick := NMoTick;
end;

function MoPressed (var Buttons, PosX, PosY): Word; assembler;
asm
  mov ax, 5
  les di, Buttons
  mov bx, word ptr es:[di]
  int 33h
  mov word ptr es:[di], ax
  les di, PosX
  mov word ptr es:[di], cx
  les di, PosY
  mov word ptr es:[di], dx
  mov ax, bx
end;

function MoReleased (var Buttons, PosX, PosY): Word; assembler;
asm
  mov ax, 6
  les di, Buttons
  mov bx, word ptr es:[di]
  int 33h
  mov word ptr es:[di], ax
  les di, PosX
  mov word ptr es:[di], cx
  les di, PosY
  mov word ptr es:[di], dx
  mov ax, bx
end;

procedure MoGetMove (var GetX, GetY); assembler;
asm
  mov ax, 0Bh
  int 33h
  les di, GetX
  mov word ptr es:[di], cx
  les di, GetY
  mov word ptr es:[di], dx
end;

procedure MoShow; assembler;
asm
  mov ax, 1
  int 33h
end;

procedure MoHide; assembler;
asm
  mov ax,2
  int 33h
end;

function MoGetPage: Word; assembler;
asm
  mov ax,1Eh
  int 33h
  mov ax, bx
end;

procedure MoSetPage (PageNo : Word); assembler;
asm
  mov ax, 1Dh
  mov bx, PageNo
  int 33h
end;

procedure MoGoToXY (X, Y: Word); assembler;
asm
  mov ax, 4
  mov cx, X
  mov dx, Y
  int 33h
end;

procedure MoSaveStatus (Buff: Pointer); assembler;
asm
  les dx, dword ptr Buff
  mov ax, 0016h
  int 33h
end;

procedure MoLoadStatus (Buff: Pointer); assembler;
asm
  les dx, dword ptr Buff
  mov ax, 0017h
  int 33h
end;

function  MoBuffSize: Word; assembler;
asm
  mov ax, 0015h
  int 33h
  mov ax, bx
end;

procedure MoExchHandler (var HandlPtr, Mask); assembler;
asm
  mov ax, 0014h
  les di, Mask
  mov cx, word ptr es:[di]
  les di, HandlPtr
  mov dx, word ptr es:[di]
  mov es, word ptr es:[di][2]
  int 33h
  mov ax, es
  les di, HandlPtr
  mov word ptr es:[di], dx
  mov word ptr es:[di][2], ax
  les di, Mask
  mov word ptr es:[di], cx
end;

end.